#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// Enter the description of your new custom indicator here
    /// </summary>
    [Description("Enter the description of your new custom indicator here")]
	[Gui.Design.DisplayName("Boll_MACD")]
    public class BollMacd : Indicator
    {
        #region Variables
        // Wizard generated variables
        private int					fast	           = 5;
		private int					slow	           = 34;
		private int					smooth	           = 5;
		private double				stDv 	           = 2;
		private int              bollLength            = 50;
		private DataSeries                            MeBo,AvgBMac;
		private int 				length 	           = 20;
		private Color		        dnColor	           = Color.Red;
		private Color		        upColor		       = Color.Chartreuse;
		private int HiLo1,HiLo2 ;
		
		 
		// User defined variables (add any user defined variables below)
	   	private DataSeries  bbMacd;
        #endregion
		
        /// <summary>
        /// This method is used to configure the strategy and is called once before any strategy method is called.
        /// </summary>
        protected override void Initialize()
        {
			 Add(new Plot(new Pen(Color.Yellow,2),    "BB_UpperBand"));
			Add(new Plot(new Pen(Color.Blue,2),       "BB_LowerBand"));
			Add(new Plot(new Pen(Color.LimeGreen,2),  "Media_BB_Up"));
			Add(new Plot(new Pen(Color.Firebrick,2),  "Media_BB_Dn"));
			Add(new Plot(new Pen(Color.AliceBlue,2),   "Media_BB"));
			
			Overlay				= false;
			DrawOnPricePanel	= false;
			CalculateOnBarClose = false;
					
			bbMacd    = new DataSeries(this);
			MeBo      = new DataSeries(this);
			AvgBMac   = new DataSeries(this);
			
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
           
		
						
			
			 if(CurrentBar > Slow + 1)
			{	
			
				
			bbMacd.Set(MACD(Input, fast, slow, smooth)[0]);
			AvgBMac.Set(EMA(bbMacd,BollLength)[0]);	
			MeBo.Set(EMA(AvgBMac,BollLength)[0]);
			double mediaBoll = (MeBo[0]);	
			double SDev = StdDev(bbMacd,BollLength)[0];	
			double upperBand = mediaBoll+(stDv*SDev);
			double lowerBand = mediaBoll-(stDv*SDev);
			double prevMacdVal= (bbMacd[1]);
			double macdVal = (bbMacd[0]);
			double zeroUp = mediaBoll;
			double zeroDn = mediaBoll;
				
			    
					
				BB_UpperBand.Set(upperBand);
   			    BB_LowerBand.Set(lowerBand);
				mediaBollMac.Set(mediaBoll);
				
			if ( macdVal > prevMacdVal)
			     HiLo1 = 1;
     else 
			if ( macdVal < prevMacdVal)
			      HiLo1 = -1;
			 if (HiLo1 == 1){
				
				
				      DrawDot(CurrentBar.ToString(),0,macdVal,UpColor);
	} else 
				if (HiLo1 == -1){
					
					
					DrawDot(CurrentBar.ToString(),0,macdVal,DnColor);
					
				}
				if  (prevMacdVal< mediaBoll && macdVal > mediaBoll)
					HiLo2 = 1;
				else
				if( prevMacdVal >= mediaBoll && macdVal < mediaBoll)
					HiLo2 = -1;
				
				if (HiLo2 == 1){
					LinezeroUp.Set(zeroUp);
				}
				else
					if (HiLo2 == -1){
					LinezeroDn.Set(zeroDn);
					}
            }
		}
        #region Properties
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries mediaBollMac
		{
			get { return Values[4]; }
		}
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries BB_UpperBand
		{
			get { return Values[0]; }
		}
		
        /// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries BB_LowerBand
		{
			get { return Values[1]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries LinezeroUp
		{
			get { return Values[2]; }
		}
		
        /// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries LinezeroDn
		{
			get { return Values[3]; }
		}
		
        /// <summary>
		/// </summary>
		[Description("Number of bars for fast EMA")]
		[Category("Parameters")]
		public int Fast
		{
			get { return fast; }
			set { fast = Math.Max(1, value); }
		}

		/// <summary>
		/// </summary>
		[Description("Number of bars for slow EMA")]
		[Category("Parameters")]
		public int Slow
		{
			get { return slow; }
			set { slow = Math.Max(1, value); }
		}

		/// <summary>
		/// </summary>
		[Description("Number of bars for smoothing")]
		[Category("Parameters")]
		public int Smooth
		{
			get { return smooth; }
			set { smooth = Math.Max(1, value); }
		}

		/// <summary>
		/// </summary>
		[Description("Standard Deviation Factor")]
		[Category("Parameters")]
		public double StDev
		{
			get { return stDv; }
			set { stDv = Math.Max(0, value); }
		}
		/// <summary>
		/// </summary>
		[Description("BollLength")]
		[Category("Parameters")]
		public int BollLength
		{
			get { return bollLength; }
			set { bollLength = Math.Max(1, value); }
		}

		/// <summary>
		/// </summary>
		[Description("Period")]
		[Category("Parameters")]
		public int Period
		{
			get { return length; }
			set { length = Math.Max(1, value); }
		}
        // Create our user definable color input
        [Description("Color for painted region")]
        [Category("Parameters")]
        public Color UpColor
        {
            get { return upColor; }
            set { upColor = value; }
        }
		
		// Serialize our Color object
		[Browsable(false)]
		public string UpColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(upColor); }
			set { upColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
	   
		// Create our user definable color input
        [Description("Color for painted region")]
        [Category("Parameters")]
        public Color DnColor
        {
            get { return dnColor; }
            set {dnColor = value; }
        }
		
		// Serialize our Color object
		[Browsable(false)]
		public string DnColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(dnColor); }
			set { dnColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
        #endregion
    }
}
#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private BollMacd[] cacheBollMacd = null;

        private static BollMacd checkBollMacd = new BollMacd();

        /// <summary>
        /// Enter the description of your new custom indicator here
        /// </summary>
        /// <returns></returns>
        public BollMacd BollMacd(int bollLength, Color dnColor, int fast, int period, int slow, int smooth, double stDev, Color upColor)
        {
            return BollMacd(Input, bollLength, dnColor, fast, period, slow, smooth, stDev, upColor);
        }

        /// <summary>
        /// Enter the description of your new custom indicator here
        /// </summary>
        /// <returns></returns>
        public BollMacd BollMacd(Data.IDataSeries input, int bollLength, Color dnColor, int fast, int period, int slow, int smooth, double stDev, Color upColor)
        {
            checkBollMacd.BollLength = bollLength;
            bollLength = checkBollMacd.BollLength;
            checkBollMacd.DnColor = dnColor;
            dnColor = checkBollMacd.DnColor;
            checkBollMacd.Fast = fast;
            fast = checkBollMacd.Fast;
            checkBollMacd.Period = period;
            period = checkBollMacd.Period;
            checkBollMacd.Slow = slow;
            slow = checkBollMacd.Slow;
            checkBollMacd.Smooth = smooth;
            smooth = checkBollMacd.Smooth;
            checkBollMacd.StDev = stDev;
            stDev = checkBollMacd.StDev;
            checkBollMacd.UpColor = upColor;
            upColor = checkBollMacd.UpColor;

            if (cacheBollMacd != null)
                for (int idx = 0; idx < cacheBollMacd.Length; idx++)
                    if (cacheBollMacd[idx].BollLength == bollLength && cacheBollMacd[idx].DnColor == dnColor && cacheBollMacd[idx].Fast == fast && cacheBollMacd[idx].Period == period && cacheBollMacd[idx].Slow == slow && cacheBollMacd[idx].Smooth == smooth && Math.Abs(cacheBollMacd[idx].StDev - stDev) <= double.Epsilon && cacheBollMacd[idx].UpColor == upColor && cacheBollMacd[idx].EqualsInput(input))
                        return cacheBollMacd[idx];

            BollMacd indicator = new BollMacd();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.BollLength = bollLength;
            indicator.DnColor = dnColor;
            indicator.Fast = fast;
            indicator.Period = period;
            indicator.Slow = slow;
            indicator.Smooth = smooth;
            indicator.StDev = stDev;
            indicator.UpColor = upColor;
            indicator.SetUp();

            BollMacd[] tmp = new BollMacd[cacheBollMacd == null ? 1 : cacheBollMacd.Length + 1];
            if (cacheBollMacd != null)
                cacheBollMacd.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheBollMacd = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Enter the description of your new custom indicator here
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.BollMacd BollMacd(int bollLength, Color dnColor, int fast, int period, int slow, int smooth, double stDev, Color upColor)
        {
            return _indicator.BollMacd(Input, bollLength, dnColor, fast, period, slow, smooth, stDev, upColor);
        }

        /// <summary>
        /// Enter the description of your new custom indicator here
        /// </summary>
        /// <returns></returns>
        public Indicator.BollMacd BollMacd(Data.IDataSeries input, int bollLength, Color dnColor, int fast, int period, int slow, int smooth, double stDev, Color upColor)
        {
            return _indicator.BollMacd(input, bollLength, dnColor, fast, period, slow, smooth, stDev, upColor);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Enter the description of your new custom indicator here
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.BollMacd BollMacd(int bollLength, Color dnColor, int fast, int period, int slow, int smooth, double stDev, Color upColor)
        {
            return _indicator.BollMacd(Input, bollLength, dnColor, fast, period, slow, smooth, stDev, upColor);
        }

        /// <summary>
        /// Enter the description of your new custom indicator here
        /// </summary>
        /// <returns></returns>
        public Indicator.BollMacd BollMacd(Data.IDataSeries input, int bollLength, Color dnColor, int fast, int period, int slow, int smooth, double stDev, Color upColor)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.BollMacd(input, bollLength, dnColor, fast, period, slow, smooth, stDev, upColor);
        }

    }
}
#endregion
